@@ -165,18 +165,18 @@ rsvg_paint_server_lin_grad_render (RsvgPaintServer *self, ArtRender *ar,
 		}
 
 	if (rlg->obj_bbox) {
-		affine[0] = ar->x1 - ar->x0;
+		affine[0] = ctx->x1 - ctx->x0;
 		affine[1] = 0.;		
 		affine[2] = 0.;
-		affine[3] = ar->y1 - ar->y0;
-		affine[4] = ar->x0;
-		affine[5] = ar->y0;
-
-		art_affine_multiply (affine, affine, rlg->affine);
+		affine[3] = ctx->y1 - ctx->y0;
+		affine[4] = ctx->x0;
+		affine[5] = ctx->y0;
 	} else {
 		for (i = 0; i < 6; i++)
-			affine[i] = rlg->affine[i];
+			affine[i] = ctx->affine[i];
 	}
+
+	art_affine_multiply(affine, rlg->affine, affine);
 	
 	/* compute [xy][12] in pixel space */
 	x1 = rlg->x1 * affine[0] + rlg->y1 * affine[2] + affine[4];
@@ -190,7 +190,7 @@ rsvg_paint_server_lin_grad_render (RsvgPaintServer *self, ArtRender *ar,
 	dy = y2 - y1;
 
 	/* workaround for an evil devide by 0 bug - not sure if this is sufficient */
-	if (fabs(dx + dy) <= 0.0000001)
+	if (fabs(dx) + fabs(dy) <= 0.0000001)
 		scale = 0.;
 	else
 		scale = 1.0 / (dx * dx + dy * dy);
@@ -239,19 +239,20 @@ rsvg_paint_server_rad_grad_render (RsvgPaintServer *self, ArtRender *ar,
 	int i;
 
 	if (rrg->obj_bbox) {
-		affine[0] = ar->x1 - ar->x0;
+		affine[0] = ctx->x1 - ctx->x0;
 		affine[1] = 0.;		
 		affine[2] = 0.;
-		affine[3] = ar->y1 - ar->y0;
-		affine[4] = ar->x0;
-		affine[5] = ar->y0;
+		affine[3] = ctx->y1 - ctx->y0;
+		affine[4] = ctx->x0;
+		affine[5] = ctx->y0;
 
-		art_affine_multiply (affine, affine, rrg->affine);
 	} else {
 		for (i = 0; i < 6; i++)
-			affine[i] = rrg->affine[i];
+			affine[i] = ctx->affine[i];
 	}
 
+	art_affine_multiply(affine, rrg->affine, affine);
+
 	agr = z->agr;
 	if (agr == NULL)
 		{
